/*
 * FFmpeg main
 * Copyright (c) 2000-2003 Fabrice Bellard
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

extern "C"
{
  #include "ffmpeg.h"
}

#include "ffmpeg_class.h"
#include "kdebug.h"

static FFMpeg* ffmpeg_instance = 0;

FFMpeg::FFMpeg()
{
  av_reset();
  ffmpeg_instance = this;
  set_progress(&FFMpeg::progress);
}

FFMpeg::~FFMpeg()
{
  ffmpeg_instance = 0;
}

int FFMpeg::transcode()
{
  m_stop = 0;
  int result = av_transcode();
  if(m_stop)
    return 1;
  return result;
}

void FFMpeg::set(const char* option, const char* value)
{
  av_set(option, value);
}

void FFMpeg::set_output(const char *filename)
{
  parse_arg_file(filename);
}

void FFMpeg::transcodeProgress(int)
{
}

void FFMpeg::progress(int frame)
{
  ffmpeg_instance->transcodeProgress(frame);
}

void FFMpeg::stop(int stop)
{
  m_stop = stop;
  av_stop(stop);
}
