//**************************************************************************
//   Copyright (C) 2004, 2005 by Petri Damstn
//   petri.damsten@iki.fi
//
//   This program is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation; either version 2 of the License, or
//   (at your option) any later version.
//
//   This program is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program; if not, write to the
//   Free Software Foundation, Inc.,
//   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//**************************************************************************

#ifndef QFFMPEG_H
#define QFFMPEG_H

#include <kdebug.h>
#include <qobject.h>
#include <qimage.h>
#include <qdatetime.h>
#include <qmap.h>
#include "libavformat/avformat.h"
#include "libavcodec/avcodec.h"

class QAVImage : public QImage
{
  public:
    QAVImage(AVFrame* frame, int width, int height);
};

class QAVTime : public QTime
{
  public:
    QAVTime() : QTime() {};
    QAVTime(QTime tm) : QTime(tm) {};
    QAVTime(int64_t timestamp);
    QAVTime(double timestamp);

    int64_t toAVTime() const;
    double  toDouble() const;
    void set(int64_t timestamp);
    void set(double timestamp);

    operator double() const { return toDouble(); };
    operator int64_t() const { return toAVTime(); };
    operator int() const { return (int)toDouble(); };
    QAVTime& operator+=(int64_t i)
      { *this = addMSecs(i / (AV_TIME_BASE / 1000)); return *this; };
    QAVTime& operator+=(double d)
      { *this = addMSecs((int)(d*1000.0)); return *this; };
    QAVTime& operator=(double d) { set(d); return *this; };
    QAVTime& operator=(int64_t i) { set(i); return *this; };
};

class QFFMPegFile
{
  public:
    QFFMPegFile(const QString& filename = QString::null);
    virtual ~QFFMPegFile();
    void reset();
    bool open(const QString& filename);
    const QString fileName() const { return m_filename; };
    const AVInputFormat* fileFormat() const
        { return (m_pFormatCtx)?m_pFormatCtx->iformat:0; };
    void freeAVData();
    bool seek(int64_t timestamp, bool backwards);
    int64_t duration() const
        { return (m_pFormatCtx)?m_pFormatCtx->duration:0; };
    AVFrame* frame() const;
    double frameRate() const;
    int64_t frameLength() const;
    int width() const;
    int height() const;
    void SaveFrame(AVFrame *pFrame, int width, int height) const;
    bool hasVideo() const { return m_videoStream >= 0; };
    bool hasAudio() const { return m_audioStreams.count() > 0; };
    QValueList<int> audioStreams() const { return m_audioStreams; };

  protected:
    QString          m_filename;
    AVFrame*         m_pFrameRGB;
    AVFrame*         m_pFrame;
    uint8_t*         m_buffer;
    AVFormatContext* m_pFormatCtx;
    AVCodecContext*  m_pCodecCtx;
    int              m_videoStream;
    QValueList<int>  m_audioStreams;
    int64_t          m_pos;
};

typedef QValueList<QFFMPegFile> QFFMPegFileList;
typedef QMap<QString, QString> QFFMpegConvertTo;

class FFMpegConverter;

class QFFMpeg : public QObject
{
    Q_OBJECT
    friend class FFMpegConverter;
  public:
    QFFMpeg();
    virtual ~QFFMpeg();

    const QFFMPegFileList& files() const { return m_files; };
    void addFile(const QString& file);
    QImage frame(QTime qtime)
        { return QAVImage(rawFrame(qtime), width(), height()); };
    AVFrame* rawFrame(QTime qtime);
    QAVTime position() const { return m_position; };
    QAVTime duration() const;
    int width() const { return m_files.first().width(); };
    int height() const { return m_files.first().height(); };
    uint audioStreamCount() const;
    QStringList fileNames() const;
    void clear();
    double frameRate() const;
    bool isDVDCompatible() const;
    bool convertTo(QFFMpegConvertTo type, const QString& file);
    QDateTime lastModified();
    void stop();

  signals:
    void convertProgress(int);

  private:
    QFFMPegFileList m_files;
    QAVTime m_position;
    FFMpegConverter* m_ff;
};

#endif // QFFMPEG_H
