//**************************************************************************
//   Copyright (C) 2004, 2005 by Petri Damstn
//   petri.damsten@iki.fi
//
//   This program is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation; either version 2 of the License, or
//   (at your option) any later version.
//
//   This program is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program; if not, write to the
//   Free Software Foundation, Inc.,
//   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//**************************************************************************
#include "font.h"
#include <fontconfig/fontconfig.h>
#include <qregexp.h>

QMap<QString, QString> KMF::Font::m_fileMap;

Magick::TypeMetric KMF::Font::typeMetrics(const QString& text) const
{
  Magick::Image background;
  Magick::TypeMetric type_metrics;
  std::string txt = text.latin1();
  background.font(m_name.ascii());
  background.fontPointsize(m_size);
  background.fontTypeMetrics(txt, &type_metrics);
  return type_metrics;
}

int KMF::Font::pixelHeight(const QString& text) const
{
  Magick::TypeMetric t = typeMetrics(text);
  int h = (int)t.textHeight();
  QRegExp re("[gjpqy]");

  if(re.search(text) == -1) // no letters that have descent
    h += (int)t.descent();
  return h;
}

int KMF::Font::pixelWidth(const QString& text) const
{
  return (int)typeMetrics(text).textWidth();
}

void KMF::Font::toXML(QDomElement& element) const
{
  element.setAttribute("name", m_name);
  //element.setAttribute("color", m_color);
  element.setAttribute("size", m_size);
  element.setAttribute("weight", m_weight);
}

void KMF::Font::fromXML(const QDomElement& element)
{
  m_name = element.attribute("name", "Helvetica");
  // Widget has color attribute
  //m_color = element.attribute("color", "0").toLong();
  m_size = element.attribute("size", "22").toInt();
  m_weight = element.attribute("weight", "400").toInt();
}

const QMap<QString, QString>& KMF::Font::fileMap()
{
  if(m_fileMap.count() == 0)
  {
    FcObjectSet* os;
    FcPattern* pat;
    FcFontSet* fontset;
    int i;

    os = FcObjectSetBuild(FC_FAMILY, FC_FILE, NULL);
    pat = FcPatternCreate();
    fontset = FcFontList(NULL, pat, os);
    FcPatternDestroy(pat);
    FcObjectSetDestroy(os);

    for(i=0; i<fontset->nfont; i++)
    {
      FcChar8* family;
      FcChar8* file;

      FcPatternGetString(fontset->fonts[i], FC_FAMILY, 0, &family);
      FcPatternGetString(fontset->fonts[i], FC_FILE, 0, &file);
      m_fileMap[(const char*)family] = (const char*)file;
    }
    FcFontSetDestroy (fontset);
  }
  return m_fileMap;
}
