//**************************************************************************
//   Copyright (C) 2004 by Petri Damstn
//   petri.damsten@iki.fi
//
//   This program is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation; either version 2 of the License, or
//   (at your option) any later version.
//
//   This program is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program; if not, write to the
//   Free Software Foundation, Inc.,
//   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//**************************************************************************
#include "kmfimageview.h"
#include <kdebug.h>
#include <qpainter.h>
#include <qscrollbar.h>

KMFImageView::KMFImageView(QWidget *parent, const char *name)
  : QScrollView(parent, name, WStaticContents | WNoAutoErase),
  m_scaled(false), m_valign(KMF::Rect::Middle), m_halign(KMF::Rect::Center)
{
}

KMFImageView::~KMFImageView()
{
}

void KMFImageView::drawContents(QPainter* p, int, int, int, int)
                                //int clipx, int clipy, int clipw, int cliph)
{
  //kdDebug() << "Imageview: " << clipx << ", " <<  clipy << ", "
  //    << clipw << ", " << cliph << ", " << endl;
  double scale = 0;

  p->eraseRect(rect());
  KMF::Rect rc(m_image.rect());
  if(m_scaled)
  {
    rc.set(rect());
    scale = (double)rc.width() / (double)m_image.rect().width();
  }
  rc.align(rect(), m_halign, m_valign);
  p->translate(horizontalScrollBar()->isHidden()?rc.x():0,
               verticalScrollBar()->isHidden()?rc.y():0);
  if(scale != 0)
    p->scale(scale, scale);
  p->drawPixmap(0, 0, m_image);//, clipx, clipy, clipw + 5, cliph + 5);
}

void KMFImageView::resizeEvent(QResizeEvent* re)
{
  QScrollView::resizeEvent(re);
  updateContents(rect());
}

void KMFImageView::setPixmap(const QPixmap& image)
{
  m_image = image;
  updateContents(rect());
}

void KMFImageView::setImage(const QImage& image)
{
  m_image.convertFromImage(image);
  updateContents(rect());
}

#include "kmfimageview.moc"
