//**************************************************************************
//   Copyright (C) 2004 by Petri Damstn
//   petri.damsten@iki.fi
//
//   This program is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation; either version 2 of the License, or
//   (at your option) any later version.
//
//   This program is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program; if not, write to the
//   Free Software Foundation, Inc.,
//   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//**************************************************************************
#include "config.h"
#include "outputplugin.h"
#include "dvdauthorobject.h"
#include "dvddirectoryobject.h"
#include "k3bobject.h"
#include <kdeversion.h>
#include <kgenericfactory.h>
#include <kaction.h>
#include <kfiledialog.h>
#include <kglobal.h>
#include <kiconloader.h>
#include <kaboutdata.h>
#include <qregexp.h>
#include <qpixmap.h>

static const char description[] =
  I18N_NOOP("Output plugin for KMediaFactory.");
static const char version[] = VERSION;
static const KAboutData about("kmediafactory_output",
                              I18N_NOOP("KMediaFactory Output"),
                              version, description, KAboutData::License_GPL,
                              "(C) 2005 Petri Damsten", 0, 0,
                              "petri.damsten@iki.fi");

typedef KGenericFactory<OutputPlugin> outputFactory;
#if KDE_IS_VERSION(3, 3, 0)
K_EXPORT_COMPONENT_FACTORY(kmediafactory_output, outputFactory(&about))
#else
K_EXPORT_COMPONENT_FACTORY(kmediafactory_output, outputFactory(about.appName()))
#endif

OutputPlugin::OutputPlugin(QObject *parent,
                           const char* name, const QStringList&) :
  KMF::Plugin(parent, name )
{
  // Initialize GUI
  setInstance(KGenericFactory<OutputPlugin>::instance());
  setXMLFile("kmediafactory_outputui.rc");
  addPreviewDVD = new KAction(i18n("Preview in Xine"), "xine",
                              CTRL+Key_X,
                              this, SLOT(slotPreviewDVD()), actionCollection(),
                              "preview_dvd");
}

void OutputPlugin::init(const QString &type)
{
  deleteChildren();
  if (type.left(3) == "DVD")
  {
    KMF::UiInterface *ui = uiInterface();
    if(ui)
    {
      DvdDirectoryObject *ddob = new DvdDirectoryObject(this);
      ui->addOutputObject(ddob);
      DvdAuthorObject *daob = new DvdAuthorObject(this);
      ui->addOutputObject(daob);
      DvdAuthorObject *k3bob = new K3bObject(this);
      ui->addOutputObject(k3bob);
    }
  }
}

void OutputPlugin::slotPreviewDVD()
{
  QString projectDir = projectInterface()->projectDir();
  KProcess xine;

  xine << "xine" << "dvd:/" + projectDir + "/DVD/VIDEO_TS";
  xine.start(KProcess::DontCare);
}

#include "outputplugin.moc"
