//**************************************************************************
//   Copyright (C) 2004 by Petri Damstn
//   petri.damsten@iki.fi
//
//   This program is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation; either version 2 of the License, or
//   (at your option) any later version.
//
//   This program is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program; if not, write to the
//   Free Software Foundation, Inc.,
//   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//**************************************************************************
#include "kmfbutton.h"
#include "kmfmenu.h"
#include "kmfmenupage.h"
#include <qdom.h>

KMFButton::KMFButton(QObject *parent, const char *name)
 : KMFWidget(parent, name)
{
  page()->addButton(this);
}

KMFButton::~KMFButton()
{
}

void KMFButton::fromXML(const QDomElement& element)
{
  KMFWidget::fromXML(element);
  m_directions[Up] = element.attribute("up");
  m_directions[Down] = element.attribute("down");
  m_directions[Left] = element.attribute("left");
  m_directions[Right] = element.attribute("right");
  m_jumpS = element.attribute("jump").upper();
}

void KMFButton::writeDvdAuthorXml(QDomText& element)
{
  QString s = "jump ";

  if (m_jump.titleSet() == KMFJump::TS_VMGM)
    s += QString("vmgm ");
  else if (m_jump.titleSet() > 0)
    s += QString("titleset %1 ").arg(m_jump.titleSet());

  if (m_jump.title() > 0)
  {
    s += QString("title %1 ").arg(m_jump.title());
    if (m_jump.chapter() > 0)
      s += QString("chapter %1 ").arg(m_jump.chapter());
  }
  else
  {
    s += QString("menu %1 ").arg(m_jump.menu());
  }
  s = " " + s.stripWhiteSpace() + "; ";
  element.setData(s);
}

bool KMFButton::parseJump()
{
  KMFMenuPage* menuPage = page();
  QPtrList<KMF::MediaObject>* mobs = m_prjIf->mediaObjects();

  if(m_jumpS == "PREV")
  {
    if(menuPage->titles() > 0 && menuPage->titleStart() > 0)
    {
      m_jump.setMenu(menuPage->titleStart() / menuPage->titles());
    }
    else if(menuPage->chapters() > 0 && menuPage->chapterStart() > 0)
    {
      m_jump.setMenu(menuPage->chapterStart() / menuPage->chapters());
    }
    else
    {
      hide();
    }
  }
  else if(m_jumpS == "NEXT")
  {
    if(menuPage->titles() > 0 &&
       (menuPage->titles() + menuPage->titleStart()) < (int)mobs->count())
    {
      m_jump.setMenu(menuPage->titleStart() / menuPage->titles() + 2);
    }
    else if(menuPage->chapters() > 0 &&
            (menuPage->chapters() + menuPage->chapterStart()) <
             mobs->at(menuPage->titleStart())->chapters())
    {
      m_jump.setMenu(menuPage->chapterStart() / menuPage->chapters() + 2);
    }
    else
    {
      hide();
    }
  }
  else if(m_jumpS == "BACK")
  {
    // TODO better back implementation
    m_jump.setMenu(1, KMFJump::TS_VMGM);
  }
  else
  {
    int title = menuPage->titleStart(), chapter = menuPage->chapterStart();
    QStringList j = QStringList::split(":", m_jumpS);

    // TODO clean these ugly ifs
    if(j[0].find(".") > -1)
    {
      parseTitleChapter(j[0], title, chapter);
      if(menuPage->titles() != 0 && title > (int)mobs->count())
      {
        hide();
        return true;
      }
      if(menuPage->chapters() != 0 &&
         chapter > mobs->at(menuPage->titleStart())->chapters())
      {
        hide();
        return true;
      }
      m_jump.setTitle(1, chapter, title);
    }
    else
    {
      if(j.count() > 1)
      {
        parseTitleChapter(j[1], title, chapter);
        if(menuPage->titles() > 0 && title > (int)mobs->count())
        {
          hide();
          return true;
        }
        if(menuPage->chapters() > 0 &&
           chapter > mobs->at(menuPage->titleStart())->chapters())
        {
          hide();
          return true;
        }
      }
      if(menu()->addPage(j[0], title, chapter) == false)
        return false;
      m_jump.setMenu(1, title);
    }
  }
  return true;
}

KMFButton* KMFButton::parseDirection(const QString& dir)
{
  QStringList d = QStringList::split(",", dir);
  KMFMenuPage* p = page();
  KMFButton* btn = 0;

  for(QStringList::Iterator it=d.begin(); it!=d.end(); ++it)
  {
    btn = p->button(*it);
    if(btn)
      return btn;
  }
  return 0;
}

void KMFButton::parseDirections()
{
  setUp(parseDirection(direction(Up)));
  setLeft(parseDirection(direction(Left)));
  setDown(parseDirection(direction(Down)));
  setRight(parseDirection(direction(Right)));
}

#include "kmfbutton.moc"
