//**************************************************************************
//   Copyright (C) 2004 by Petri Damstn
//   petri.damsten@iki.fi
//
//   This program is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation; either version 2 of the License, or
//   (at your option) any later version.
//
//   This program is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program; if not, write to the
//   Free Software Foundation, Inc.,
//   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//**************************************************************************
#include "kmflabel.h"
#include "kmfmenu.h"
#include "kmfmenupage.h"
#include <kmediafactory/rect.h>
#include <kdebug.h>
#include <qcolor.h>
#include <qregexp.h>
#include <Magick++.h>
#include <string>

KMFLabel::KMFLabel(QObject *parent, const char *name) :
  KMFWidget(parent, name), m_text("")
{
}

KMFLabel::~KMFLabel()
{
}

int KMFLabel::minimumPaintWidth() const
{
  return m_font.pixelWidth(m_text);
}

int KMFLabel::minimumPaintHeight() const
{
  return m_font.pixelHeight(m_text);
}

void KMFLabel::fromXML(const QDomElement& element)
{
  KMFWidget::fromXML(element);
  QDomNode n = element.firstChild();
  while(!n.isNull())
  {
    QDomElement e = n.toElement();
    if(!e.isNull())
    {
      if(e.tagName() == "font")
        m_font.fromXML(e);
      else if(e.tagName() == "text")
        setText(e.text());
    }
    n = n.nextSibling();
  }
}

void KMFLabel::setText(const QString& text)
{
  QRegExp rx("%([\\d\\.$]+)%" );
  QPtrList<KMF::MediaObject>* mobs = m_prjIf->mediaObjects();
  int title, chapter;
  QString s;
  int pos = 0;
  const KMFTemplate& tmplate = menu()->templateStore();

  m_text = tmplate.translate(text);

  while(pos >= 0)
  {
    pos = rx.search(text, pos);
    if(pos > -1)
    {
      parseTitleChapter(rx.cap(1), title, chapter);

      if(title < 1)
        s = m_prjIf->title();
      else
      {
        if(title <= (int)mobs->count()
          && chapter <= (int)mobs->at(title-1)->chapters())
          s = mobs->at(title-1)->text(chapter);
        else
          s = "";
      }
      m_text.replace("%" + rx.cap(1) + "%", s);
      pos += rx.matchedLength();
    }
  }
  m_text.replace("%%", "%");
}

void KMFLabel::paintWidget(Magick::Image& layer, bool shdw)
{
  QRect rc = (shdw)? paintRect(shadow().offset()) : paintRect();
  KMF::Color rgb = (shdw)? shadow().color() : color() ;
  KMF::Rect textrc(0, 0, minimumPaintWidth(), minimumPaintHeight());
  std::list<Magick::Drawable> drawList;

  drawList.push_back(Magick::DrawableStrokeWidth(0));
  drawList.push_back(Magick::DrawableGravity(Magick::NorthWestGravity));
  drawList.push_back(Magick::DrawableFillColor(rgb));
  drawList.push_back(Magick::DrawableFillOpacity((double)rgb.alpha() / 255.0));
  // Is it this one or...
  drawList.push_back(Magick::DrawableFont(m_font.name().ascii(),
                                          Magick::NormalStyle, m_font.weight(),
                                          Magick::NormalStretch));
  drawList.push_back(Magick::DrawablePointSize(m_font.size()));
  // This one. I'll do both at the moment...
  layer.font(m_font.name().ascii());
  layer.fontPointsize(m_font.size());

  std::string text = m_text.ascii();
  textrc.align(rc, halign(), valign());
  /*
  Magick::TypeMetric type_metrics;
  QString fn = m_font.name();
  std::string f = fn.ascii();
  layer.font(f);
  layer.fontPointsize(m_font.size());
  layer.fontTypeMetrics(text, &type_metrics);
  */
  drawList.push_back(Magick::DrawableText(textrc.x(), textrc.y(), text));
  //drawList.push_back(Magick::DrawableText(100, 100, text));
  //drawList.push_back(Magick::DrawableTranslation(center.x(), center.y()));
  layer.draw(drawList);
}

#include "kmflabel.moc"
