//**************************************************************************
//   Copyright (C) 2004 by Petri Damstn
//   petri.damsten@iki.fi
//
//   This program is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation; either version 2 of the License, or
//   (at your option) any later version.
//
//   This program is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program; if not, write to the
//   Free Software Foundation, Inc.,
//   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//**************************************************************************
#include "kmflayout.h"
#include <kdebug.h>
#include <qobjectlist.h>

KMFVBox::KMFVBox(QObject *parent, const char *name)
  : KMFWidget(parent, name)
{
}

KMFVBox::~KMFVBox()
{
}

int KMFVBox::minimumPaintWidth() const
{
  int result = 0;

  for(QObjectListIt it(*children()); it.current() != 0; ++it)
    result = QMAX(static_cast<KMFWidget*>(*it)->minimumWidth(), result);
  return result;
}

int KMFVBox::minimumPaintHeight() const
{
  int result = 0;

  for(QObjectListIt it(*children()); it.current() != 0; ++it)
    result += static_cast<KMFWidget*>(*it)->minimumHeight();
  return result;
}

int KMFVBox::childY(const KMFWidget* child) const
{
  int result = paintY();

  for(QObjectListIt it(*children()); it.current() != 0; ++it)
  {
    KMFWidget* widget = static_cast<KMFWidget*>(*it);
    if(widget == child)
      return result;
    result += widget->height();
  }
  return result;
}

int KMFVBox::childHeight() const
{
  int result = 0;

  for(QObjectListIt it(*children()); it.current() != 0; ++it)
  {
    KMFWidget* widget = static_cast<KMFWidget*>(*it);
    if(widget->geometry().height().type() != KMFUnit::Percentage)
      result += static_cast<KMFWidget*>(*it)->height();
  }
  return paintHeight() - result;
}

KMFHBox::KMFHBox(QObject *parent, const char *name)
  : KMFWidget(parent, name)
{
}

KMFHBox::~KMFHBox()
{
}

int KMFHBox::minimumPaintWidth() const
{
  int result = 0;

  for(QObjectListIt it(*children()); it.current() != 0; ++it)
  {
    KMFWidget* widget = static_cast<KMFWidget*>(*it);
    //kdDebug() << widget->className() << ": "
    //    << widget->minimumWidth() << endl;
    result += widget->minimumWidth();
  }
  return result;
}

int KMFHBox::minimumPaintHeight() const
{
  int result = 0;

  for(QObjectListIt it(*children()); it.current() != 0; ++it)
  {
    KMFWidget* widget = static_cast<KMFWidget*>(*it);
    //kdDebug() << widget->className() << ": "
    //    << widget->minimumHeight() << endl;
    result = QMAX(widget->minimumHeight(), result);
  }
  return result;
}

int KMFHBox::childX(const KMFWidget* child) const
{
  int result = paintX();

  for(QObjectListIt it(*children()); it.current() != 0; ++it)
  {
    KMFWidget* widget = static_cast<KMFWidget*>(*it);
    if(widget == child)
      return result;
    result += widget->width();
  }
  return result;
}

int KMFHBox::childWidth() const
{
  int result = 0;

  for(QObjectListIt it(*children()); it.current() != 0; ++it)
  {
    KMFWidget* widget = static_cast<KMFWidget*>(*it);
    if(widget->geometry().width().type() != KMFUnit::Percentage)
      result += static_cast<KMFWidget*>(*it)->width();
  }
  return paintWidth() - result;
}

#include "kmflayout.moc"
