//**************************************************************************
//   Copyright (C) 2004 by Petri Damstn
//   petri.damsten@iki.fi
//
//   This program is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation; either version 2 of the License, or
//   (at your option) any later version.
//
//   This program is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program; if not, write to the
//   Free Software Foundation, Inc.,
//   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//**************************************************************************
#include "kmfmenu.h"
#include "kmfwidgetfactory.h"
#include "QMImage.h"
#include <kstore/koStore.h>
#include <kdebug.h>
#include <klocale.h>
#include <qimage.h>
#include <qfile.h>
#include <qfileinfo.h>

KMFMenu::KMFMenu(const QString& tmplate, QObject *parent, const char *name)
  : KMFTemplateBase(parent, name),
    m_template(tmplate), m_templateXML("kmf_project")
{
  m_pages.setAutoDelete(true);
  m_templateXML.setContent(m_template.readFile("template.xml"));
  QDomElement element = m_templateXML.documentElement();
  m_id = element.attribute("id");
  m_title = element.attribute("name");
}

KMFMenu::~KMFMenu()
{
}

bool KMFMenu::writeDvdAuthorXml(QDomDocument& doc)
{
  QDomElement root = doc.createElement("dvdauthor");
  uint i;

  root.setAttribute("jumppad", "1");
  root.setAttribute("dest", "./DVD");
  doc.appendChild(root);

  QDomElement elem = doc.createElement("vmgm");
  QDomElement menus = doc.createElement("menus");

  if(pages() > 0)
  {
    for(KMFMenuPageIterator it(*m_pages[0]); *it != 0; ++it)
      (*it)->writeDvdAuthorXml(menus);
    elem.appendChild(menus);
    root.appendChild(elem);

    KMF::MediaObject *mob;
    QPtrList<KMF::MediaObject>* mobs = m_prjIf->mediaObjects();

    for (i=1, mob = mobs->first(); mob; mob = mobs->next(), ++i)
    {
      QDomElement elem = doc.createElement("titleset");
      QDomElement menus = doc.createElement("menus");
      if(i < m_pages.count() && m_pages[i] != 0)
      {
        for(KMFMenuPageIterator it(*m_pages[i]); *it != 0; ++it)
          (*it)->writeDvdAuthorXml(menus);
      }
      elem.appendChild(menus);
      mob->writeDvdAuthorXml(elem);
      root.appendChild(elem);
    }
    return true;
    //kdDebug() << doc.toString() << endl;
  }
  else
  {
    m_uiIf->message(KMF::Error, i18n("No pages."));
    return false;
  }
}

bool KMFMenu::writeDvdAuthorXml(const QString& fileName)
{
  QDomDocument doc("");
  if(writeDvdAuthorXml(doc))
  {
    QFile file(fileName);
    if (!file.open(IO_WriteOnly))
      return false;
    QTextStream stream(&file);
    stream << doc.toString();
    file.close();
    return true;
  }
  return false;
}

bool KMFMenu::makeMenuMpegs(QString projectPath)
{
  for(uint i=0; i < m_pages.count(); ++i)
  {
    if(m_pages[i])
    {
      for(KMFMenuPageIterator it(*m_pages[i]); *it != 0; ++it)
      {
        progress(m_pagePoints);
        if(m_uiIf->message(KMF::Info,
           QString(i18n("   Menu: %1")).arg(uiText((*it)->name()))))
          return false;

        (*it)->makeMpeg(projectPath);
      }
    }
  }
  return true;
}

QPtrList<KMFMenuPage>* KMFMenu::titlePages(uint title)
{
  if(m_pages.count() <= title)
    m_pages.resize(title + 1);
  if(m_pages[title] == 0)
  {
    m_pages.insert(title, new QPtrList<KMFMenuPage>);
    m_pages[title]->setAutoDelete(true);
  }
  return m_pages[title];
}

bool KMFMenu::addPage(const QDomElement& element, uint pageSet,
                     uint title, uint chapter)
{
  KMFMenuPage* menuPage = static_cast<KMFMenuPage*>
      (KMFWidgetFactory::createPage(element, this, title, chapter));
  progress(m_pagePoints);
  if(menuPage)
  {
    if(m_uiIf->message(KMF::Info,
       QString(i18n("   Menu: %1")).arg(uiText(menuPage->name()))))
      return false;
    titlePages(pageSet)->append(menuPage);
    return menuPage->parseButtons();
  }
  return false;
}

bool KMFMenu::addPage(const QString& name, uint title, uint chapter)
{
  QDomElement element = m_templateXML.documentElement();
  QDomElement pageElement = getPage(element.firstChild(), name);
  QPtrList<KMF::MediaObject>* mobs = m_prjIf->mediaObjects();
  KMFMenuPage temp(this);

  temp.fromXML(pageElement);

  if(temp.titles() > 0)
  {
    for(uint i = 0; i < ((mobs->count() - 1) / temp.titles()) + 1; ++i)
    {
      if(addPage(pageElement, title, i * temp.titles(), chapter) == false)
        return false;
    }
  }
  else if(temp.chapters() > 0)
  {
    if(title <= mobs->count())
    {
      for(int i = 0;
          i < ((mobs->at(title-1)->chapters() - 1) / temp.chapters()) + 1;
          ++i)
      {
        if(addPage(pageElement, title, title - 1, i * temp.chapters()) == false)
          return false;
      }
    }
  }
  else
  {
    return addPage(pageElement, title, title, chapter);
  }
  return true;
}

QDomElement KMFMenu::getPage(const QDomNode& node, const QString& name)
{
  QDomNode n = node;
  while(!n.isNull())
  {
    QDomElement e = n.toElement();
    if(!e.isNull())
    {
      if(e.tagName() == "page" && e.attribute("name") == name)
      {
        return e;
      }
    }
    n = n.nextSibling();
  }
  m_uiIf->message(KMF::Error,
                  i18n("Cannot find page %1 from template.").arg(name));
  return QDomElement();
}

QImage KMFMenu::templateImage(const QString& image) const
{
  return QImage(m_template.readFile(image));
}

QImage KMFMenu::makeMenuPreview(QString page)
{
  if(page.isEmpty())
  {
    return templateImage("preview.jpg");
  }
  else
  {
    QDomElement element = m_templateXML.documentElement();
    QDomElement e = getPage(element.firstChild(), page);
    KMFMenuPage* p =
        static_cast<KMFMenuPage*>(KMFWidgetFactory::createPage(e, this));
    if(p)
    {
      p->paint();
      // Generating previews here. comment this for release.
      // kdDebug() << "WRITING PREVIEW. DISABLE BEFORE RELEASE." << endl;
      // p->layer(KMFMenuPage::Background).write("preview.jpg");
      return QMImage(p->layer(KMFMenuPage::Background)).image();
    }
  }
  return QImage();
}

int KMFMenu::pages()
{
  int result = 0;

  for(uint i=0; i < m_pages.count(); ++i)
  {
    if(m_pages[i])
    {
      for(KMFMenuPageIterator it(*m_pages[i]); *it != 0; ++it)
      {
        ++result;
      }
    }
  }
  return result;
}

void KMFMenu::progress(int points)
{
  if(points > m_points)
    points = m_points;
  m_points -= points;
  m_uiIf->progress(points);
}

bool KMFMenu::makeMenu()
{
  m_points = TotalPoints / 4;
  m_pagePoints = m_points / ((m_prjIf->mediaObjects()->count() * 2) + 1);
  QDomElement element = m_templateXML.documentElement();
  QString page = element.attribute("first_page");

  if(m_uiIf->message(KMF::Info, i18n("Generating menus")))
    return false;
  if(addPage(page, 0, 0))
  {
    int p = pages();
    progress(TotalPoints);
    m_points = TotalPoints - (TotalPoints / 4);
    if(p > 0)
      m_pagePoints = m_points / p;
    // Generate jpgs
    m_uiIf->message(KMF::Info, i18n("Making menu mpegs"));
    if(makeMenuMpegs(m_prjIf->projectDir()) == false)
      return false;

    // Generate xml
    if(m_uiIf->message(KMF::Info, i18n("Generating DVDAuthor xml")))
      return false;
    bool result = writeDvdAuthorXml(m_prjIf->projectDir() + "/dvdauthor.xml");
    progress(TotalPoints);
    clear();
    return result;
  }
  return false;
}

#include "kmfmenu.moc"
