//**************************************************************************
//   Copyright (C) 2004 by Petri Damstn
//   petri.damsten@iki.fi
//
//   This program is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation; either version 2 of the License, or
//   (at your option) any later version.
//
//   This program is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program; if not, write to the
//   Free Software Foundation, Inc.,
//   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//**************************************************************************
#include "templateobject.h"
#include "templateplugin.h"
#include "templateoptions.h"
#include "kmfmenu.h"
#include "QMImage.h"
#include "kmediafactory/dvdauthorparser.h"
#include <kmimetype.h>
#include <kapplication.h>
#include <kurl.h>
#include <klocale.h>
#include <kaction.h>
#include <kdebug.h>
#include <kstandarddirs.h>
#include <kiconloader.h>
#include <qptrlist.h>
#include <qdom.h>
#include <qfile.h>
#include <qfileinfo.h>
#include <qimage.h>

TemplateObject::TemplateObject(const QString& templateFile, QObject* parent):
  KMF::TemplateObject(parent), m_menu(templateFile, this)
{
  setTitle(m_menu.title());
  setName(m_menu.id().ascii());
  m_templateProperties = new KAction(i18n("&Properties"), "pencil", 0, this,
      SLOT(slotProperties()), plugin()->actionCollection(), "tob_properties");
}

void TemplateObject::actions(QPtrList<KAction>& actionList) const
{
  actionList.append(m_templateProperties);
}

void TemplateObject::fromXML(const QDomElement& element)
{
  QDomNode n = element.firstChild();
  while(!n.isNull())
  {
    QDomElement e = n.toElement();
    if(!e.isNull())
    {
      if(e.tagName() == "custom_properties")
      {
        QDomNode n2 = e.firstChild();
        while(!n2.isNull())
        {
          QDomElement e2 = n2.toElement();
          if(!e2.isNull())
          {
            if(e2.tagName() == "properties")
            {
              QString widget = e2.attribute("widget");
              QDomNode n3 = e2.firstChild();
              while(!n3.isNull())
              {
                QDomElement e3 = n3.toElement();
                if(!e3.isNull())
                {
                  if(e3.tagName() == "property")
                  {
                    setProperty(widget, e3.attribute("name"),
                                QVariant(e3.attribute("value")));
                  }
                }
                n3 = n3.nextSibling();
              }
            }
          }
          n2 = n2.nextSibling();
        }
      }
    }
    n = n.nextSibling();
  }
}

void TemplateObject::toXML(QDomElement& element) const
{
  QDomDocument doc = element.ownerDocument();
  QDomElement tmplate = doc.createElement("custom_properties");

  for(KMFPropertiesDictIterator it(m_customProperties); *it != 0; ++it)
  {
    QDomElement e = doc.createElement("properties");
    e.setAttribute("widget", it.currentKey());
    for(KMFPropertiesIterator jt(*(*it)); *jt != 0; ++jt)
    {
      QDomElement e2 = doc.createElement("property");
      e2.setAttribute("name", jt.currentKey());
      e2.setAttribute("value", (*jt)->toString());
      e.appendChild(e2);
    }
    tmplate.appendChild(e);
  }
  element.appendChild(tmplate);
}

int TemplateObject::timeEstimate() const
{
  return KMFMenu::TotalPoints;
}

bool TemplateObject::make()
{
  // Make menu
  if(isUpToDate())
  {
    uiInterface()->message(KMF::Info, i18n("Menus are up to date"));
    uiInterface()->progress(KMFMenu::TotalPoints);
    return true;
  }
  else
    return m_menu.makeMenu();
}

QStringList TemplateObject::menus()
{
  // TODO: return menu list
  return QStringList();
}

QImage TemplateObject::preview(const QString& menu)
{
  return m_menu.makeMenuPreview(menu);
}

QPixmap TemplateObject::pixmap() const
{
  return m_menu.icon();
}

void TemplateObject::slotProperties()
{
  TemplateOptions dlg(kapp->activeWindow());
  dlg.setData(*this);
  if (dlg.exec())
  {
    dlg.getData(*this);
    projectInterface()->setDirty();
  }
}

QVariant TemplateObject::property(const QString& widget,
                                  const QString& name) const
{
  KMFProperties* prop = m_customProperties.find(widget);
  if(prop)
  {
    QVariant* dict = prop->find(name);
    if(dict)
      return *dict;
  }
  return QVariant();
}

void TemplateObject::setProperty(const QString& widget,
                                 const QString& name, const QVariant& value)
{
  KMFProperties* prop = m_customProperties.find(widget);
  if(!prop)
    m_customProperties.insert(widget, new KMFProperties);
  m_customProperties[widget]->replace(name, new QVariant(value));
}

bool TemplateObject::isUpToDate()
{
  QDateTime lastModified = projectInterface()->lastModified();
  QString file = projectInterface()->projectDir() + "/dvdauthor.xml";
  QFileInfo fileInfo(file);

  if(fileInfo.exists() == false || lastModified > fileInfo.lastModified())
    return false;

  KMF::DVDAuthorParser da;
  da.setFile(file);
  QStringList files = da.files();

  if(files.count() < 1)
    return false;

  for(QStringList::Iterator it = files.begin(); it != files.end(); ++it)
  {
    if((*it).startsWith("./menus/"))
    {
      fileInfo.setFile(projectInterface()->projectDir() + "/" + *it);

      if(fileInfo.exists() == false || lastModified > fileInfo.lastModified())
        return false;
    }
  }
  return true;
}

#include "templateobject.moc"
