//**************************************************************************
//   Copyright (C) 2004 by Petri Damstn
//   petri.damsten@iki.fi
//
//   This program is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation; either version 2 of the License, or
//   (at your option) any later version.
//
//   This program is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program; if not, write to the
//   Free Software Foundation, Inc.,
//   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//**************************************************************************
#include "templateoptions.h"
#include <kmediafactory/kmflanguages.h>
#include <kurlrequester.h>
#include <kcolorcombo.h>

TemplateOptions::TemplateOptions(QWidget *parent, const char *name)
 : TemplateOptionsLayout(parent, name)
{
}

TemplateOptions::~TemplateOptions()
{
}

void TemplateOptions::getData(TemplateObject& obj) const
{
  QStringList languages = obj.menu().templateStore().languages();

  obj.setProperty("background", "url", QVariant(backgroundURL->url()));
  obj.setProperty("label_.*", "color", QVariant(colorCombo->color()));
  obj.setProperty("%KMFMenuPage%", "language",
    QVariant(languages[languageCombo->currentItem()]));
}

void TemplateOptions::setData(const TemplateObject& obj)
{
  KURL url = obj.property("background", "url").toString();
  QStringList languages = obj.menu().templateStore().languages();
  KMFLanguages languageStrings;
  int i = 0;

  if(url.isLocalFile())
    backgroundURL->setURL(url.path());
  else
    backgroundURL->setURL(url.prettyURL());
  colorCombo->setColor(obj.property("label_.*", "color").toColor());
  for(QStringList::ConstIterator it = languages.begin();
      it != languages.end(); ++it)
  {
    QString language = languageStrings[*it];
    if(language.isEmpty())
      language = languageStrings["en"];
    languageCombo->insertItem(language);
    if(*it == obj.property("%KMFMenuPage%", "language").toString())
      languageCombo->setCurrentItem(i);
    ++i;
  }
}

#include "templateoptions.moc"
