//**************************************************************************
//   Copyright (C) 2004, 2005 by Petri Damstn
//   petri.damsten@iki.fi
//
//   This program is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation; either version 2 of the License, or
//   (at your option) any later version.
//
//   This program is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program; if not, write to the
//   Free Software Foundation, Inc.,
//   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//**************************************************************************

#include "chapters.h"
#include "videoplugin.h"
#include "kmfchapterlistview.h"
#include "autochapterslayout.h"
#include <qffmpeg.h>
#include <kmediafactory/kmfimageview.h>
#include <kmediafactory/plugin.h>
#include <kapplication.h>
#include <kmainwindow.h>
#include <kstdaction.h>
#include <kinputdialog.h>
#include <klocale.h>
#include <kmessagebox.h>
#include <kdebug.h>
#include <ktimewidget.h>
#include <qpopupmenu.h>
#include <qslider.h>
#include <qimage.h>
#include <qlabel.h>
#include <qlineedit.h>

Chapters::Chapters(QWidget *parent, const char *name)
  : ChaptersLayout(parent, name), m_obj(0)
{
  video->setScaled(true);
}

Chapters::~Chapters()
{
}

void Chapters::getData(VideoOptions& obj) const
{
  KMFChapterListViewItem* item = 0;

  obj.chapters().clear();

  for(QListViewItemIterator it(chaptersListView); *it != 0; ++it)
  {
    item = static_cast<KMFChapterListViewItem*>(*it);
    KMFChapter* chapter = new KMFChapter;
    chapter->time = item->pos();
    chapter->name = item->name();
    obj.chapters().append(chapter);
  }
}

void Chapters::setData(const VideoOptions& obj)
{
  for(KMFChapterListIterator chapter(obj.chapters()); *chapter != 0; ++chapter)
  {
    new KMFChapterListViewItem(chaptersListView, chaptersListView->lastItem(),
                               (*chapter)->name, (*chapter)->time);
  }
  m_obj = obj.mob();
  chaptersListView->setDuration(m_obj->duration());
  timeSlider->setMaxValue((int)m_obj->duration());
  m_duration = KMFChapterListViewItem::time2String(m_obj->duration());
  m_pos = 0.0;
  updateVideo();

  /* TODO: Why this DOESN'T work ?????????????
  KMF::Plugin* plugin = m_obj->plugin();
  new KAction(i18n("&Delete all"), "editdelete", CTRL+Key_Delete,
      this, SLOT(deleteAll()), plugin->actionCollection(), "delete_all");
  new KAction(i18n("&Rename all"), "", KShortcut(), this,
      SLOT(renameAll()), plugin->actionCollection(), "rename_all" );
  new KAction(i18n("&Auto chapters"), "", KShortcut(), this,
      SLOT(autoChapters()), plugin->actionCollection(), "auto_chapters");
  new KAction(i18n("&Import"), "", KShortcut(), this,
      SLOT(import()), plugin->actionCollection(), "import" );
  */
}

void Chapters::updateVideo()
{
  QImage img = m_obj->getFrame(m_pos);
  video->setImage(img);

  QString s = QString("%1: %2 / %3").
      arg(m_obj->text()).
      arg(KMFChapterListViewItem::time2String(m_pos)).
      arg(m_duration);
  timeLabel->setText(s);

  timeSlider->setValue((int)m_pos);
}

void Chapters::slotSliderMoved(int value)
{
  m_pos = (double)value;
  updateVideo();
}

void Chapters::moveFrames(int direction)
{
  double move = (double)((double)direction * (1.0 / m_obj->frameRate()));
  m_pos += move;
  if(m_pos < QAVTime(0.0))
    m_pos = 0.0;
  else if(m_pos > m_obj->duration())
    m_pos = m_obj->duration();
  updateVideo();
}

void Chapters::slotNextFrame()
{
  moveFrames(1);
}

void Chapters::slotPrevFrame()
{
  moveFrames(-1);
}

void Chapters::slotForward()
{
  moveFrames(120);
}

void Chapters::slotRewind()
{
  moveFrames(-120);
}

void Chapters::slotStart()
{
  m_pos = 0.0;
  updateVideo();
}

void Chapters::slotEnd()
{
  m_pos = m_obj->duration();
  updateVideo();
}

void Chapters::slotSelectionChanged()
{
  if(chaptersListView->selectedItem())
  {
    m_pos = chaptersListView->selectedItem()->pos();
    updateVideo();
  }
}

void Chapters::slotRemove()
{
  delete chaptersListView->selectedItem();
}

void Chapters::slotAdd()
{
  bool ok;

  QString text = KInputDialog::getText(
      "KMediaFactory", i18n("Chapter name:"), QString::null, &ok, this);

  if(ok && !text.isEmpty())
  {
    KMFChapterListViewItem* item = 0;

    for(QListViewItemIterator it(chaptersListView); *it != 0; ++it)
    {
      item = static_cast<KMFChapterListViewItem*>(*it);
      if((double)m_pos < item->pos())
        break;
    }
    if(item)
      item = static_cast<KMFChapterListViewItem*>(item->itemAbove());
    new KMFChapterListViewItem(chaptersListView, item, text, m_pos);
  }
}

void Chapters::slotContextMenu(KListView*, QListViewItem*, const QPoint& p)
{
  /* TODO: Why this DOESN'T work ?????????????
  KMF::Plugin* plugin = m_obj->plugin();
  KXMLGUIFactory* factory = plugin->factory();
  QPopupMenu *popup =
      static_cast<QPopupMenu*>(factory->container("chapter_popup", plugin));
  if(popup)
    popup->exec(p);
  */
  QPopupMenu *popup = new QPopupMenu( this );
  popup->insertItem(i18n("&Delete all"), this, SLOT(deleteAll()));
  popup->insertItem(i18n("&Rename all"), this, SLOT(renameAll()));
  popup->insertItem(i18n("&Auto chapters"), this, SLOT(autoChapters()));
  // TODO: what kind of files to import?
  //popup->insertItem(i18n("&Import"), this, SLOT(import()));
  popup->exec(p);
}

void Chapters::renameAll()
{
  AutoChaptersLayout dlg(kapp->activeWindow());
  dlg.intervalLabel->hide();
  dlg.intervalTime->hide();
  dlg.resize(dlg.minimumSize());
  if (dlg.exec())
  {
    QString text = dlg.nameEdit->text();
    int i = 1;

    for(QListViewItemIterator it(chaptersListView); *it != 0; ++it)
    {
      KMFChapterListViewItem* item = static_cast<KMFChapterListViewItem*>(*it);
      item->setName(QString(text).arg(i++));
    }
  }
}

void Chapters::deleteAll()
{
  chaptersListView->clear();
}

void Chapters::autoChapters()
{
  AutoChaptersLayout dlg(kapp->activeWindow());
  if (dlg.exec())
  {
    QString text = dlg.nameEdit->text();
    double interval = (double)dlg.intervalTime->time().msecsTo(QTime())/-1000.0;
    double time = 0.0;
    int i = 1;

    if(interval < 1.0)
      return;
    chaptersListView->clear();
    while(time < (double)m_obj->duration())
    {
      new KMFChapterListViewItem(chaptersListView,
                                chaptersListView->lastItem(),
                                QString(text).arg(i), time);
      time += interval;
      ++i;
    }
  }
}

void Chapters::import()
{
  KMessageBox::error(kapp->activeWindow(),
                     i18n("XXX Not implemented yet. Kino file import here???"));
}

#include "chapters.moc"
