//**************************************************************************
//   Copyright (C) 2005 by Petri Damstn
//   petri.damsten@iki.fi
//
//   Based heavily on dv2sub.c
//
//   Copyright (C) Miguel Sahagun
//
//   Parts of this code from
//   recode.c (libdv)  Copyright (C) Dan Dennedy, Peter Schlaile
//
//   This program is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation; either version 2 of the License, or
//   (at your option) any later version.
//
//   This program is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program; if not, write to the
//   Free Software Foundation, Inc.,
//   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//**************************************************************************
#include "dvobject.h"
#include "videopluginsettings.h"
#include "progresslayout.h"
#include <kapplication.h>
#include <kpushbutton.h>
#include <kdebug.h>
#include <qprogressbar.h>
#include <qdir.h>
#include <qeventloop.h>
#include <libdv/dv.h>
#include <libdv/dv_types.h>

DVObject::DVObject(QObject* parent)
  : VideoObject(parent)
{
}

DVObject::~DVObject()
{
}

void DVObject::cancel()
{
  m_cancel = true;
}

int readFrame(QFile& video, char *frameBuffer, bool& isPAL)
{
  if(video.readBlock(frameBuffer, 120000) != 120000)
  {
    return 0;
  }

  isPAL = (frameBuffer[3] & 0x80);

  if (isPAL)
  {
    if (video.readBlock(frameBuffer + 120000, 144000 - 120000) !=
        144000 - 120000)
    {
      return 0;
    }
  }
  return 1;
}

bool DVObject::addFile(QString fileName)
{
  VideoObject::addFile(fileName);

  QDir dir(projectInterface()->projectDir() + "/media/");
  QFileInfo fi(fileName);
  QFile sub(dir.filePath(id() + ".sub"));
  QFile in(fileName);
  uint8_t dvBuffer[144000];
  // assume NTSC for now, switch to PAL later if needed
  dv_decoder_t *decoder = dv_decoder_new(FALSE, FALSE, FALSE);
  int i = 0;
  bool isPAL = false;
  int lastframe = 1;
  i = 0;
  int framenum = 0;
  struct tm recDate;
  QDateTime date;
  QDateTime lastDate(QDate(1800, 1, 1));
  QDateTime lastChapter(QDate(1800, 1, 1));
  QString subFormat = VideoPluginSettings::subtitleFormat();
  int minChapLength = VideoPluginSettings::minChapterLength();
  int chapterOffset = VideoPluginSettings::chapterOffset();
  ProgressLayout progressDlg;

  if(!dir.exists())
    dir.mkdir(dir.path());

  // Clear automatically generated chapters
  m_chapters.clear();
  m_cancel = false;
  // Gets two audio channels???
  m_audioStreams.clear();
  m_audioStreams.append("en");

  progressDlg.show();
  connect(progressDlg.cancelButton, SIGNAL(clicked()),
          SLOT(cancel()));
  int frames = (int)(duration().toDouble() * frameRate());
  progressDlg.progressBar->setTotalSteps(frames);

  if(decoder)
  {
    if(in.open(IO_ReadOnly))
    {
      if(sub.open(IO_WriteOnly))
      {
        QTextStream subStream(&sub);

        while(readFrame(in, (char*)dvBuffer, isPAL) && !m_cancel)
        {
          ++framenum;
          // dv_parse_header(decoder, dvBuffer);
          dv_parse_packs(decoder, dvBuffer);
          dv_get_recording_datetime_tm(decoder, &recDate);
          /*
          kdDebug()
              << " : " << recDate.tm_year
              << " : " << recDate.tm_mon
              << " : " << recDate.tm_mday
              << " : " << recDate.tm_hour
              << " : " << recDate.tm_min
              << " : " << recDate.tm_sec << endl;
          */
          date.setTime_t(mktime(&recDate));
          // False dates ?
          if(date < lastDate && date.time() == QTime(0,0))
            continue;
          // New take
          if(date.addSecs(-1 * minChapLength) > lastChapter)
          {
            lastChapter = date;
            KMFChapter* c = new KMFChapter;
            if(framenum > 1)
              c->time = (double)(framenum + chapterOffset) / frameRate();
            else
              c->time = 0.0;
            c->name = date.toString(subFormat);
            m_chapters.append(c);
          }
          if(framenum == 1)
            lastDate = date;

          progressDlg.progressBar->setProgress(framenum);
          kapp->eventLoop()->processEvents(QEventLoop::AllEvents);
          // New second or take
          if(lastDate.toString(subFormat) != date.toString(subFormat))
          {
            lastDate = date;
            subStream << "{" << lastframe << "}{" << framenum - 1
                << "} " << lastDate.toString(subFormat) << endl;
            lastframe = framenum;
          }
        }
        subStream << "{" << lastframe << "}{" << framenum
            << "} " << lastDate.toString(subFormat) << endl;
        sub.close();
        addSubtitle(sub.name(), "", KMF::Rect::Bottom, KMF::Rect::Right);
      }
      in.close();
    }
    dv_decoder_free(decoder);
  }
  return true;
}

#include "dvobject.moc"
