//**************************************************************************
//   Copyright (C) 2004 by Petri Damstn
//   petri.damsten@iki.fi
//
//   This program is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation; either version 2 of the License, or
//   (at your option) any later version.
//
//   This program is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program; if not, write to the
//   Free Software Foundation, Inc.,
//   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//**************************************************************************
#ifndef VIDEOOBJECT_H
#define VIDEOOBJECT_H

#include <qffmpeg.h>
#include <kmediafactory/plugin.h>
#include <kmediafactory/rect.h>
#include <kmediafactory/font.h>
#include <kurl.h>
#include <kprocess.h>
#include <qptrlist.h>
#include <qdatetime.h>

struct KMFChapter
{
  QString name;
  double time;
};

class KMFChapterList : public QPtrList<KMFChapter>
{
  virtual Item newItem(Item item)
  {
    KMFChapter* old_item = static_cast<KMFChapter*>(item);
    KMFChapter* new_item = new KMFChapter;
    new_item->name = old_item->name;
    new_item->time = old_item->time;
    return new_item;
  }
};

typedef QPtrListIterator< KMFChapter > KMFChapterListIterator;

class KMFSubtitle
{
  public:
    KMFSubtitle();

    KURL file;
    QString language;
    KMF::Font font;
    KMF::Rect::VAlign valign;
    KMF::Rect::HAlign halign;
};

typedef QPtrList< KMFSubtitle > KMFSubtitleList;
typedef QPtrListIterator< KMFSubtitle > KMFSubtitleListIterator;

/**
*/
class VideoObject : public KMF::MediaObject
{
    Q_OBJECT
  public:
    VideoObject(QObject* parent);
    virtual ~VideoObject();
    virtual void toXML(QDomElement& element) const;
    virtual void fromXML(const QDomElement& element);
    virtual void writeDvdAuthorXml(QDomElement& element);
    virtual bool make();
    virtual int timeEstimate() const;
    virtual QPixmap pixmap() const;
    virtual QImage preview(int chapter = MainPreview) const;
    virtual QString text(int chapter = MainTitle) const;
    virtual int chapters() const;
    const KMFChapterList& chapterList() const { return m_chapters; };
    void setChapterList(const KMFChapterList& list) { m_chapters = list; };
    virtual void actions(QPtrList<KAction>& actionList) const;
    virtual const QStringList& audioStreams() const { return m_audioStreams; };
    void setAudioStreams(const QStringList& streams)
        { m_audioStreams = streams; };
    virtual KMF::AspectRatio aspect() const { return m_aspect; };
    void setAspect(KMF::AspectRatio aspect) { m_aspect = aspect; };

    virtual bool addFile(QString fileName);
    void setTitleFromFileName();
    const KURL& previewURL() const { return m_previewURL; };
    void setPreviewURL(const KURL& previewURL) { m_previewURL = previewURL; };
    QImage getFrame(double time) const;
    const QString& id() const { return m_id; };
    void clearSubtitles() { m_subtitles.clear(); };
    void addSubtitle(const KURL& file, const QString& language = "",
                     KMF::Rect::VAlign valign = KMF::Rect::Bottom,
                     KMF::Rect::HAlign halign = KMF::Rect::Left,
                     const KMF::Font& font = KMF::Font());
    const KMFSubtitleList& subtitles() const { return m_subtitles; };

    double frameRate() const { return m_decoder.frameRate(); };
    QAVTime duration() const { return m_decoder.duration(); };
    virtual QStringList files() const {  return m_decoder.fileNames(); };
    QString fileName() const { return m_decoder.fileNames().first(); };

  public slots:
    virtual void slotProperties();
    virtual void slotProgress(int);

  private slots:
    void output(KProcess* proc, char* buffer, int buflen);

  protected:
    mutable KMFChapterList m_chapters;
    QStringList m_audioStreams;
    KMFSubtitleList m_subtitles;

  private:
    mutable QFFMpeg m_decoder;
    //mutable QFFMpeg m_tmpDecoder;
    KAction* m_videoProperties;
    KURL m_previewURL;
    KMF::AspectRatio m_aspect;
    QString m_id;
    QString m_buffer;
    QTime m_lastUpdate;
    bool m_stopped;

    void VideoObject::generateId();
    void setChapterSecs(double secs);
    bool isBlack(const QImage& img) const;
    bool convertToDVD(QString& fileName);
    bool convertSubtitles(QString& fileName);
    bool writeSpumuxXml(QDomDocument& doc);
    bool writeSpumuxXml(const QString& fileName);
    QString VideoObject::checkFontFile(const QString& file);
};

#endif // VIDEOOBJECT_H
