//**************************************************************************
//   Copyright (C) 2004 by Petri Damstn
//   petri.damsten@iki.fi
//
//   This program is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation; either version 2 of the License, or
//   (at your option) any later version.
//
//   This program is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program; if not, write to the
//   Free Software Foundation, Inc.,
//   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//**************************************************************************
#include "videooptions.h"
#include "chapters.h"
#include "languageselectionlayout.h"
#include <kapplication.h>
#include <klocale.h>
#include <kdebug.h>
#include <kurlrequester.h>
#include <ktimewidget.h>
#include <qlineedit.h>
#include <qcombobox.h>

class KMFLanguageListBoxItem : public QListBoxText
{
  public:
    KMFLanguageListBoxItem(QListBox* listbox,
                           const QString& text = QString::null,
                           const QString& language = QString::null)
      : QListBoxText(listbox, text), m_language(language) {};

    const QString& language() const { return m_language; };
    void setLanguage(const QString& language) { m_language = language; };
    void set(const QString& text, const QString& language)
        { m_language = language; setText(text); listBox()->updateContents(); };

  private:
    QString m_language;;
};

VideoOptions::VideoOptions(QWidget *parent, const char *name)
  : VideoOptionsLayout(parent, name)
{
}

VideoOptions::~VideoOptions()
{
}

void VideoOptions::setData(const VideoObject& obj)
{
  titleEdit->setText(obj.title());
  m_chapters = obj.chapterList();
  previewURL->setURL(obj.previewURL().prettyURL());
  aspectComboBox->setCurrentItem((int)obj.aspect());

  // TODO: support for multiple subtitles
  KMFSubtitleList subs = obj.subtitles();
  if(subs.count() > 0)
    subtitleURL->setURL(subs.at(0)->file.prettyURL());

  const QStringList& audioStreams = obj.audioStreams();
  int i = 1;
  for(QStringList::ConstIterator it = audioStreams.begin();
      it != audioStreams.end(); ++it)
  {
    QString language = m_languages[*it];
    if(language.isEmpty())
      language = m_languages["en"];
    new KMFLanguageListBoxItem(audioListBox,
                               QString("%1. %2").arg(i).arg(language),
                               *it);
  }
  audioListBox->setSelected(0, true);
  m_obj = &obj;
}

void VideoOptions::getData(VideoObject& obj) const
{
  obj.setTitle(titleEdit->text());
  obj.setChapterList(m_chapters);
  obj.setPreviewURL(KURL(previewURL->url()));
  obj.setAspect((KMF::AspectRatio)aspectComboBox->currentItem());

  obj.clearSubtitles();
  if(!subtitleURL->url().isEmpty())
    obj.addSubtitle(subtitleURL->url());
  QStringList languages;
  KMFLanguageListBoxItem* item = static_cast<KMFLanguageListBoxItem*>
      (audioListBox->firstItem());
  while(item)
  {
    languages.append(item->language());
    item = static_cast<KMFLanguageListBoxItem*>(item->next());
  }
  obj.setAudioStreams(languages);
}

void VideoOptions::audioClicked()
{
  LanguageSelectionLayout dlg(this);
  KMFLanguageListBoxItem* selected = static_cast<KMFLanguageListBoxItem*>
      (audioListBox->selectedItem());

  QStringList keys = m_languages.keys();
  for(QStringList::ConstIterator it = keys.begin(); it != keys.end(); ++it)
  {
    KMFLanguageListBoxItem* item = new KMFLanguageListBoxItem(
        dlg.languageListBox, m_languages[*it], *it);
    if(*it == selected->language())
      dlg.languageListBox->setSelected(item, true);
  }
  dlg.languageListBox->ensureCurrentVisible();
  if (dlg.exec())
  {
    QString lang = static_cast<KMFLanguageListBoxItem*>
        (dlg.languageListBox->selectedItem())->language();
    selected->set(QString("%1.%2").
        arg(audioListBox->index(selected) +1).
        arg(m_languages[lang]), lang);
  }
}

void VideoOptions::chaptersClicked()
{
  Chapters dlg(this);
  dlg.setData(*this);
  if (dlg.exec())
  {
    dlg.getData(*this);
  }
}

#include "videooptions.moc"
