//**************************************************************************
//   Copyright (C) 2004 by Petri Damstn
//   petri.damsten@iki.fi
//
//   This program is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation; either version 2 of the License, or
//   (at your option) any later version.
//
//   This program is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program; if not, write to the
//   Free Software Foundation, Inc.,
//   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//**************************************************************************
#include "config.h"
#include "videoplugin.h"
#include "videoobject.h"
#include "dvobject.h"
#include <avcodec.h>
#include <kgenericfactory.h>
#include <kdeversion.h>
#include <kaction.h>
#include <kfiledialog.h>
#include <kglobal.h>
#include <kiconloader.h>
#include <kaboutdata.h>
#include <qregexp.h>
#include <qpixmap.h>

static const char description[] =
  I18N_NOOP("Video plugin for KMediaFactory.");
static const char version[] = VERSION;
static const KAboutData about("kmediafactory_video",
                              I18N_NOOP("KMediaFactory Video"),
                              version, description, KAboutData::License_GPL,
                              "(C) 2005 Petri Damsten", 0, 0,
                              "petri.damsten@iki.fi");

typedef KGenericFactory<VideoPlugin> videoFactory;
#if KDE_IS_VERSION(3, 3, 0)
K_EXPORT_COMPONENT_FACTORY(kmediafactory_video, videoFactory(&about))
#else
K_EXPORT_COMPONENT_FACTORY(kmediafactory_video, videoFactory(about.appName()))
#endif

// ffmpeg callbacks doesn't have context pointers
KMF::Logger* g_logger = 0;

VideoPlugin::VideoPlugin(QObject *parent,
                         const char* name, const QStringList&) :
  KMF::Plugin(parent, name )
{
  // Initialize GUI
  setInstance(KGenericFactory<VideoPlugin>::instance());
  setXMLFile("kmediafactory_videoui.rc");
  // Add action for menu item
  addVideoAction = new KAction(i18n("Add Video"), "video", CTRL+Key_V,
                               this, SLOT(slotAddVideo()), actionCollection(),
                               "video");
  addDVAction = new KAction(i18n("Add DV"), "camera_unmount", CTRL+Key_D,
                               this, SLOT(slotAddDV()), actionCollection(),
                               "dv");
}

void VideoPlugin::ffmpeg_av_log_callback(void *ptr, int level,
                                         const char *fmt, va_list vl)
{
  if(level > av_log_get_level())
    return;

  QString prefix;

  if(ptr)
  {
    AVClass* avc= *(AVClass**)ptr;
    if(avc)
      prefix.sprintf("[%s @ %p]", avc->item_name(ptr), avc);
  }
  if(g_logger)
    g_logger->message(prefix, fmt, vl);
}

void VideoPlugin::init(const QString &type)
{
  deleteChildren();
  if (type.left(3) == "DVD")
  {
    addVideoAction->setEnabled(true);
  }
  else
  {
    addVideoAction->setEnabled(false);
  }
  g_logger = this->uiInterface()->logger();
  av_log_set_callback(VideoPlugin::ffmpeg_av_log_callback);
}

void VideoPlugin::slotAddVideo()
{
  QStringList filenames =
    KFileDialog::getOpenFileNames(":AddVideo",
                                  "*.mpg *.mpeg *.vob|MPEG video files");
  KMF::UiInterface *m = uiInterface();
  if(m && filenames.count() > 0)
  {
    VideoObject *vob = new VideoObject(this);
    for(QStringList::ConstIterator filename = filenames.begin();
        filename != filenames.end(); ++filename)
      vob->addFile(*filename);
    vob->setTitleFromFileName();
    m->addMediaObject(vob);
  }
}

void VideoPlugin::slotAddDV()
{
  QStringList filenames =
      KFileDialog::getOpenFileNames(":AddDV",
                                    "*.dv |DV video files");
  KMF::UiInterface *m = uiInterface();
  if(m && filenames.count() > 0)
  {
    DVObject *vob = new DVObject(this);
    for(QStringList::ConstIterator filename = filenames.begin();
        filename != filenames.end(); ++filename)
      vob->addFile(*filename);
    vob->setTitleFromFileName();
    m->addMediaObject(vob);
  }
}

KMF::MediaObject* VideoPlugin::createMediaObject(QDomElement& element)
{
  KMF::MediaObject *mob = new VideoObject(this);
  if(mob)
    mob->fromXML(element);
  return mob;
}

#include "videoplugin.moc"
