//**************************************************************************
//   Copyright (C) 2004 by Petri Damstn
//   petri.damsten@iki.fi
//
//   This program is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation; either version 2 of the License, or
//   (at your option) any later version.
//
//   This program is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program; if not, write to the
//   Free Software Foundation, Inc.,
//   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//**************************************************************************
#include "kmfapplication.h"
#include "kmfuiinterface.h"
#include "kmfprojectinterface.h"
#include "kmediafactory.h"
#include <qobjectlist.h>
#include <kcmdlineargs.h>
#include <ktrader.h>
#include <kparts/componentfactory.h>
#include <kdebug.h>
#include <kmainwindow.h>

KMFApplication::KMFApplication()
 : KApplication(), m_mainWin(0), m_project(0)
{
}

KMFApplication::~KMFApplication()
{
}

void KMFApplication::init()
{
  KCmdLineArgs *args = KCmdLineArgs::parsedArgs();

  loadPlugins();
  if(args->count() > 0)
    m_url = args->url(0);

  m_mainWin = new KMediaFactory();
  setMainWidget(m_mainWin);
  m_mainWin->show();
  args->clear();
}

KMFProject *KMFApplication::newProject()
{
  if(m_project)
    delete m_project;
  m_project = new KMFProject(this);
  return m_project;
}

void KMFApplication::initPlugins()
{
  const QObjectList *list = m_pluginInterface->children();
  QObject *obj;

  for(QObjectListIt it(*list); (obj=it.current())!= 0; ++it)
    if(obj->inherits("KMF::Plugin"))
      ((KMF::Plugin*)obj)->init(m_project->type());
}

void KMFApplication::loadPlugins()
{
  m_pluginInterface = new QObject(this, "pluginInterface");
  m_uiInterface = new KMFUiInterface(m_pluginInterface,
                                     "ui interface");
  m_projectInterface = new KMFProjectInterface(m_pluginInterface,
                                               "project interface");

  KTrader::OfferList offers = KTrader::self()->query("KMediaFactory/Plugin");
  KTrader::OfferList::ConstIterator iter;

  for(iter = offers.begin(); iter != offers.end(); ++iter)
  {
    int error = 0;
    KService::Ptr service = *iter;
    KMF::Plugin* plugin =
        KParts::ComponentFactory::createInstanceFromService<KMF::Plugin>
        (service, m_pluginInterface, service->name(), QStringList(), &error);
    if (plugin)
    {
      kdDebug() << "Loaded plugin " << plugin->name() << endl;
    }
    else
    {
      switch (error)
      {
        case KParts::ComponentFactory::ErrNoServiceFound:
          kdDebug() << "No service implementing the given "
                    << "mimetype and fullfilling the given constraint "
                    << "expression can be found."
                    << endl;
          break;

        case KParts::ComponentFactory::ErrServiceProvidesNoLibrary:
          kdDebug() << "the specified service provides no shared library."
                    << endl;
          break;

        case KParts::ComponentFactory::ErrNoLibrary:
          kdDebug() << "the specified library could not be loaded." << endl;
          break;

        case KParts::ComponentFactory::ErrNoFactory:
          kdDebug() << "the library does not export a factory for creating "
                    << "components."
                    << endl;
          break;

        case KParts::ComponentFactory::ErrNoComponent:
          kdDebug() << "the factory does not support creating components of "
                    << "the specified type."
                    << endl;
          break;
      }
      kdDebug() << "Loading plugin '" << service->name()
                << "' failed, KLibLoader reported error: '" << endl
                << KLibLoader::self()->lastErrorMessage()
                << "'" << endl;
    }
  }
}

