//**************************************************************************
//   Copyright (C) 2004 by Petri Damstn
//   petri.damsten@iki.fi
//
//   This program is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation; either version 2 of the License, or
//   (at your option) any later version.
//
//   This program is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program; if not, write to the
//   Free Software Foundation, Inc.,
//   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//**************************************************************************
#include "outputpage.h"
#include "kmficonview.h"
#include "kmediafactory.h"
#include "kmfapplication.h"
#include "kmfprogresslistview.h"
#include "kmfprogressitem.h"
#include "kmfuiinterface.h"
#include "logview.h"
#include <kcursor.h>
#include <kpushbutton.h>
#include <kprogress.h>
#include <klistview.h>
#include <klocale.h>
#include <qpopupmenu.h>
#include <qheader.h>

OutputPage::OutputPage(QWidget *parent, const char *name) :
  OutputPageLayout(parent, name)
{
  connect(outputs, SIGNAL(currentChanged(QIconViewItem*)),
          this, SLOT(currentChanged(QIconViewItem*)));
  connect(outputs,
          SIGNAL(contextMenuRequested(QIconViewItem*, const QPoint&)),
          this, SLOT(contextMenuRequested(QIconViewItem*, const QPoint&)));
}

OutputPage::~OutputPage()
{
}

void OutputPage::currentChanged(QIconViewItem* item)
{
  KMFIconViewItem* it = static_cast<KMFIconViewItem*>(item);
  KMF::OutputObject* ob = static_cast<KMF::OutputObject*>(it->ob());

  kmfApp->project()->setOutput(ob);
}

void OutputPage::contextMenuRequested(QIconViewItem *item, const QPoint &pos)
{
  if(item)
  {
    KMediaFactory* mainWindow =
        static_cast<KMediaFactory*>(kmfApp->mainWidget());
    KXMLGUIFactory* factory = mainWindow->factory();
    KMF::Object* ob = (static_cast<KMFIconViewItem*>(item))->ob();

    QPtrList<KAction> actions;
    ob->actions(actions);
    factory->plugActionList(mainWindow,
        QString::fromLatin1("output_actionlist"), actions);
    QWidget *w = factory->container("output_popup", mainWindow);
    if(w)
    {
      QPopupMenu *popup = static_cast<QPopupMenu*>(w);
      popup->exec(pos);
    }
    factory->unplugActionList(mainWindow, "ouput_actionlist");
  }
}

void OutputPage::showLog()
{
  LogView dlg(this);
  dlg.setData(kmfApp->logger());
  dlg.exec();
}

void OutputPage::stop()
{
  stopPushBtn->setEnabled(false);
  kmfApp->uiInterface()->setStopped(true);
  kmfApp->uiInterface()->message(KMF::Error, i18n("User cancelled."));
  kmfApp->uiInterface()->setItemProgress(-1);
}

void OutputPage::start()
{
#if RECORD_TIMINGS
  stopWatch.start();
#endif
  kmfApp->mainWindow()->enableUi(false);
  showLogPushBtn->setEnabled(false);
  stopPushBtn->setEnabled(true);
  startPushBtn->setEnabled(false);
  //kmfApp->setOverrideCursor(KCursor::waitCursor());
  kmfApp->uiInterface()->setUseMessageBox(false);
  kmfApp->uiInterface()->setStopped(false);
  progressBar->setTotalSteps(kmfApp->project()->timeEstimate());
  progressBar->setValue(0);
  progressListView->clear();
  kmfApp->logger().start();
  if(kmfApp->project()->make() == false)
    if(!kmfApp->project()->error().isEmpty())
      kmfApp->uiInterface()->message(KMF::Error, kmfApp->project()->error());
  kmfApp->logger().stop();
  //kmfApp->restoreOverrideCursor();
  showLogPushBtn->setEnabled(true);
  stopPushBtn->setEnabled(false);
  startPushBtn->setEnabled(true);
  kmfApp->mainWindow()->enableUi(true);
#if RECORD_TIMINGS
  stopWatch.stop();
#endif
}

void OutputPage::aboutToShowPage(QWidget* page)
{
  if (parent() == page)
  {
    startPushBtn->setEnabled(kmfApp->project()->mediaObjects()->count() > 0);
    progressBar->setValue(0);
  }
}

#include "outputpage.moc"
