//**************************************************************************
//   Copyright (C) 2004 by Petri Damstn
//   petri.damsten@iki.fi
//
//   This program is free software; you can redistribute it and/or modify
//   it under the terms of the GNU General Public License as published by
//   the Free Software Foundation; either version 2 of the License, or
//   (at your option) any later version.
//
//   This program is distributed in the hope that it will be useful,
//   but WITHOUT ANY WARRANTY; without even the implied warranty of
//   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//   GNU General Public License for more details.
//
//   You should have received a copy of the GNU General Public License
//   along with this program; if not, write to the
//   Free Software Foundation, Inc.,
//   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//**************************************************************************
#include "templatepage.h"
#include "kmficonview.h"
#include "kmediafactory.h"
#include "kmfapplication.h"
#include <kmediafactory/kmfimageview.h>
#include "kmfuiinterface.h"
#include <kcursor.h>
#include <kdebug.h>
#include <qtimer.h>
#include <qcheckbox.h>
#include <qlabel.h>
#include <qpopupmenu.h>

TemplatePage::TemplatePage(QWidget *parent, const char *name) :
  TemplatePageLayout(parent, name)
{
  connect(templates, SIGNAL(currentChanged(QIconViewItem*)),
          this, SLOT(currentChanged(QIconViewItem*)));
  connect(templates,
          SIGNAL(contextMenuRequested(QIconViewItem*, const QPoint&)),
          this, SLOT(contextMenuRequested(QIconViewItem*, const QPoint&)));
  templatePreview->setVScrollBarMode(QScrollView::Auto);
  templatePreview->setHScrollBarMode(QScrollView::Auto);
}

TemplatePage::~TemplatePage()
{
}

void TemplatePage::currentChanged(QIconViewItem* item)
{
  KMFIconViewItem* it = static_cast<KMFIconViewItem*>(item);
  KMF::TemplateObject* ob = static_cast<KMF::TemplateObject*>(it->ob());

  updatePreview();
  kmfApp->project()->setTemplateObj(ob);
}

void TemplatePage::aboutToShowPage(QWidget* page)
{
  if (parent() == page && templatePreview->pixmap().size() == QSize(0,0))
    QTimer::singleShot(0, this, SLOT(updatePreview()));
}

void TemplatePage::updatePreview()
{
  KMFIconViewItem* item =
      static_cast<KMFIconViewItem*>(templates->currentItem());
  if(item)
  {
    QString menu;

    kmfApp->setOverrideCursor(KCursor::waitCursor());
    kmfApp->uiInterface()->setUseMessageBox(true);
    kmfApp->uiInterface()->setStopped(false);
    if(item && kmfApp->project()->mediaObjects()->count() > 0 &&
      previewCheckBox->isChecked())
    {
      // TODO: get menu list from plugin and select name from popup menu
      menu = "Main";
    }
    KMF::TemplateObject* ob = static_cast<KMF::TemplateObject*>(item->ob());
    QPixmap pixmap = ob->preview(menu);
    templatePreview->setPixmap(pixmap);
    templatePreview->resizeContents(pixmap.width(), pixmap.height());
    kmfApp->uiInterface()->setUseMessageBox(false);
    kmfApp->restoreOverrideCursor();
  }
}

void TemplatePage::contextMenuRequested(QIconViewItem *item, const QPoint &pos)
{
  if(item)
  {
    KMediaFactory* mainWindow =
        static_cast<KMediaFactory*>(kmfApp->mainWidget());
    KXMLGUIFactory* factory = mainWindow->factory();
    KMF::Object* ob = (static_cast<KMFIconViewItem*>(item))->ob();

    QPtrList<KAction> actions;
    ob->actions(actions);
    factory->plugActionList(mainWindow,
        QString::fromLatin1("template_actionlist"), actions);
    QWidget *w = factory->container("template_popup", mainWindow);
    if(w)
    {
      QPopupMenu *popup = static_cast<QPopupMenu*>(w);
      if(popup->exec(pos) != -1)
        updatePreview();
    }
    factory->unplugActionList(mainWindow, "template_actionlist");
  }
}

void TemplatePage::previewClicked()
{
  updatePreview();
}

#include "templatepage.moc"
